@@ -1,75 +1,75 @@
 #' Interactive Plotting for Mixed Hidden Markov Model (mhmm)
 #'
-#' Function \code{plot.mhmm} plots a directed graph of the parameters of each model
+#' Function `plot.mhmm` plots a directed graph of the parameters of each model
 #' with pie charts of emission probabilities as vertices/nodes.
 #'
 #' @export
 #'
-#' @param x A hidden Markov model object of class \code{mhmm} created with
-#'   \code{\link{build_mhmm}} (or \code{\link{build_mmm}} or
-#'   \code{\link{build_lcm}}). Multichannel
-#'   \code{mhmm} objects are automatically transformed into single-channel objects.
-#'   See function \code{\link{mc_to_sc}} for more information on the
+#' @param x A hidden Markov model object of class `mhmm` created with
+#'   [build_mhmm()] (or [build_mmm()] or
+#'   [build_lcm()]). Multichannel
+#'   `mhmm` objects are automatically transformed into single-channel objects.
+#'   See function [mc_to_sc()] for more information on the
 #'   transformation.
 #' @param interactive Whether to plot each cluster in succession or in a grid.
-#'   Defaults to \code{TRUE}, i.e. clusters are plotted one after another.
+#'   Defaults to `TRUE`, i.e. clusters are plotted one after another.
 #'
-#' @param ask If \code{TRUE} and \code{which.plots} is NULL,
-#'   \code{plot.mhmm} operates in interactive mode, via \code{\link{menu}}.
-#'   Defaults to \code{FALSE}. Ignored if \code{interactive = FALSE}.
+#' @param ask If `TRUE` and `which.plots` is NULL,
+#'   `plot.mhmm` operates in interactive mode, via [utils::menu()].
+#'   Defaults to `FALSE`. Ignored if `interactive = FALSE`.
 #' @param which.plots The number(s) of the requested cluster(s) as an integer
-#'   vector. The default \code{NULL} produces all plots.
+#'   vector. The default `NULL` produces all plots.
 #'
 #' @param nrow,ncol Optional arguments to arrange plots in a grid. Ignored if
-#'   \code{interactive = TRUE}.
-#' @param byrow Controls the order of plotting in a grid. Defaults to \code{FALSE},
-#'   i.e. plots are arranged column-wise. Ignored if \code{interactive = TRUE}.
+#'   `interactive = TRUE`.
+#' @param byrow Controls the order of plotting in a grid. Defaults to `FALSE`,
+#'   i.e. plots are arranged column-wise. Ignored if `interactive = TRUE`.
 #' @param row.prop Sets the proportions of the row heights of the grid. The default
-#'   value is \code{"auto"} for even row heights. Takes a vector of values from
-#'   0 to 1, with values summing to 1. Ignored if \code{interactive = TRUE}.
+#'   value is `"auto"` for even row heights. Takes a vector of values from
+#'   0 to 1, with values summing to 1. Ignored if `interactive = TRUE`.
 #' @param col.prop Sets the proportion of the column heights of the grid. The default
-#'   value is \code{"auto"} for even column widths. Takes a vector of values
-#'   from 0 to 1, with values summing to 1. Ignored if \code{interactive = TRUE}.
+#'   value is `"auto"` for even column widths. Takes a vector of values
+#'   from 0 to 1, with values summing to 1. Ignored if `interactive = TRUE`.
 #'
 #' @param layout specifies the layout of vertices (nodes). Accepts a
-#'   numerical matrix, a \code{\link[igraph]{layout_}} function (without quotation marks),
-#'   or either of the predefined options \code{"horizontal"} (the
-#'   default) and \code{"vertical"}. Options \code{"horizontal"} and
-#'   \code{"vertical"} position vertices at the same horizontal or vertical
+#'   numerical matrix, a [igraph::layout_()] function (without quotation marks),
+#'   or either of the predefined options `"horizontal"` (the
+#'   default) and `"vertical"`. Options `"horizontal"` and
+#'   `"vertical"` position vertices at the same horizontal or vertical
 #'   line. A two-column numerical matrix can be used to give x and y coordinates of
-#'   the vertices. The \code{\link[igraph]{layout_}} functions available in the
-#'   \code{igraph} package offer other automatic layouts for graphs.
+#'   the vertices. The [igraph::layout_()] functions available in the
+#'   `igraph` package offer other automatic layouts for graphs.
 #' @param pie Are vertices plotted as pie charts of emission probabilities?
 #'   Defaults to TRUE.
 #' @param vertex.size Size of vertices, given as a scalar or numerical
 #'   vector. The default value is 40.
 #' @param vertex.label Labels for vertices. Possible options include
-#'   \code{"initial.probs"}, \code{"names"}, \code{NA}, and a character or
-#'   numerical vector. The default \code{"initial.probs"} prints the initial
-#'   probabilities of the model and \code{"names"} prints the names of the
-#'   hidden states as labels. \code{NA} prints no labels.
+#'   `"initial.probs"`, `"names"`, `NA`, and a character or
+#'   numerical vector. The default `"initial.probs"` prints the initial
+#'   probabilities of the model and `"names"` prints the names of the
+#'   hidden states as labels. `NA` prints no labels.
 #' @param vertex.label.dist Distance of the label of the vertex from its
-#'   center. The default value \code{"auto"} places the label outside the
+#'   center. The default value `"auto"` places the label outside the
 #'   vertex.
 #' @param vertex.label.pos Positions of vertex labels, relative to
 #'   the center of the vertex. A scalar or numerical vector giving
-#'   position(s) as radians or one of \code{"bottom"} (\code{pi/2} as radians),
-#'   \code{"top"} (\code{-pi/2}), \code{"left"} (\code{pi}), or
-#'   \code{"right"} (\code{0}).
+#'   position(s) as radians or one of `"bottom"` (`pi/2` as radians),
+#'   `"top"` (`-pi/2`), `"left"` (`pi`), or
+#'   `"right"` (`0`).
 #' @param vertex.label.family,edge.label.family Font family to be used for
-#'   vertex/edge labels. See argument \code{family} in \code{\link{par}} for
+#'   vertex/edge labels. See argument `family` in [par()] for
 #'   more information.
 #' @param loops Defines whether transitions back to same states are plotted.
 #' @param edge.curved Defines whether to plot curved edges (arcs, arrows)
 #'   between vertices. A logical or numerical vector or scalar. Numerical
-#'   values specify curvatures of edges. The default value \code{TRUE}
-#'   gives curvature of 0.5 to all edges. See \code{\link{igraph.plotting}} for
+#'   values specify curvatures of edges. The default value `TRUE`
+#'   gives curvature of 0.5 to all edges. See [igraph::igraph.plotting()] for
 #'   more information.
 #' @param edge.label Labels for edges. Possible options include
-#'   \code{"auto"}, \code{NA}, and a character or numerical vector. The
-#'   default \code{"auto"} prints transition probabilities as edge labels.
-#'   \code{NA} prints no labels.
-#' @param edge.width Width(s) for edges. The default \code{"auto"} determines
+#'   `"auto"`, `NA`, and a character or numerical vector. The
+#'   default `"auto"` prints transition probabilities as edge labels.
+#'   `NA` prints no labels.
+#' @param edge.width Width(s) for edges. The default `"auto"` determines
 #'   widths according to transition probabilities between hidden states.
 #'   Other possibilities are a scalar or a numerical vector of widths.
 #' @param cex.edge.width An expansion factor for edge widths. Defaults to 1.
@@ -77,7 +77,7 @@
 #' @param label.signif Rounds labels of model parameters to specified number
 #'   of significant digits, 2 by default. Ignored for user-given labels.
 #' @param label.scientific Defines if scientific notation should be used to
-#'   describe small numbers. Defaults to \code{FALSE}, e.g. 0.0001 instead of
+#'   describe small numbers. Defaults to `FALSE`, e.g. 0.0001 instead of
 #'   1e-04. Ignored for user-given labels.
 #' @param label.max.length Maximum number of digits in labels of model
 #'   parameters. Ignored for user-given labels.
@@ -88,15 +88,15 @@
 #'   value is 0.05.
 #' @param combined.slice.color Color of the combined slice that includes
 #'   the smallest emission probabilities (only if argument
-#'   \code{"combine.slices"} is greater than 0). The default color is white.
+#'   `"combine.slices"` is greater than 0). The default color is white.
 #' @param combined.slice.label The label for combined states (when argument
-#'   \code{"combine.slices"} is greater than 0) to appear in the legend.
+#'   `"combine.slices"` is greater than 0) to appear in the legend.
 #' @param with.legend Defines if and where the legend of state colors is
-#'   plotted. Possible values include \code{"bottom"} (the default),
-#'   \code{"top"}, \code{"left"}, and \code{"right"}. \code{FALSE} omits the
+#'   plotted. Possible values include `"bottom"` (the default),
+#'   `"top"`, `"left"`, and `"right"`. `FALSE` omits the
 #'   legend.
 #' @param ltext Optional description of (combined) observed states to appear
-#'   in the legend. A vector of character strings. See \code{\link{seqplot}} for
+#'   in the legend. A vector of character strings. See [TraMineR::seqplot()] for
 #'   more information.
 #' @param legend.prop Proportion used for plotting the legend. A scalar between
 #'   0 and 1, defaults to 0.5.
@@ -104,21 +104,21 @@
 #'   labels in the legend. The default value is 1. Values lesser than 1 will
 #'   reduce the size of the font, values greater than 1 will increase the size.
 #' @param ncol.legend The number of columns for the legend. The default value
-#'   \code{"auto"} sets the number of columns automatically.
+#'   `"auto"` sets the number of columns automatically.
 #' @param cpal Optional color palette for (combinations of) observed states.
-#'   The default value \code{"auto"} uses automatic color palette. Otherwise a
-#'   vector of length \code{x$n_symbols} is given, i.e. the argument requires a color
+#'   The default value `"auto"` uses automatic color palette. Otherwise a
+#'   vector of length `x$n_symbols` is given, i.e. the argument requires a color
 #'   specified for all (combinations of) observed states even if they are not
-#'   plotted (if the probability is less than \code{combine.slices}).
-#' @param main Optional main titles for plots. The default \code{"auto"} uses
-#' \code{cluster_names} as titles, \code{NULL} prints no titles.
-#' @param withlegend Deprecated. Use \code{with.legend} instead.
-#' @param ... Other parameters passed on to \code{\link{plot.igraph}} such as
-#'   \code{vertex.color}, \code{vertex.label.cex}, or \code{edge.lty}.
+#'   plotted (if the probability is less than `combine.slices`).
+#' @param main Optional main titles for plots. The default `"auto"` uses
+#' `cluster_names` as titles, `NULL` prints no titles.
+#' @param withlegend Deprecated. Use `with.legend` instead.
+#' @param ... Other parameters passed on to [igraph::plot.igraph()] such as
+#'   `vertex.color`, `vertex.label.cex`, or `edge.lty`.
 #'
-#' @seealso \code{\link{build_mhmm}} and \code{\link{fit_model}} for building and
-#'   fitting mixture hidden Markov models; \code{\link{plot.igraph}} for plotting
-#'   directed graphs; and \code{\link{mhmm_biofam}} and \code{\link{mhmm_mvad}} for
+#' @seealso [build_mhmm()] and [fit_model()] for building and
+#'   fitting mixture hidden Markov models; [igraph::plot.igraph()] for plotting
+#'   directed graphs; and [mhmm_biofam()] and [mhmm_mvad()] for
 #'   the models used in examples.
 #'
 #' @references Helske S. and Helske J. (2019). Mixture Hidden Markov Models for Sequence Data: The seqHMM Package in R,
@@ -173,7 +173,6 @@ plot.mhmm <- function(x, interactive = TRUE,
                       with.legend = "bottom", ltext = NULL, legend.prop = 0.5,
                       cex.legend = 1, ncol.legend = "auto", cpal = "auto",
                       main = "auto", withlegend, ...) {
-  check_deprecated_args(match.call())
 
   if (interactive) {
     do.call(mHMMplotint, c(list(
