@@ -53,14 +53,14 @@ mnhmm::mnhmm(
   gamma_A(gamma_A),
   gamma_B(gamma_B),
   gamma_omega(gamma_omega),
-  log_py(S, Ti(0), D),
+  py(S, Ti.max(), D),
   pi(D),
-  log_pi(D),
   A(D),
-  log_A(D),
   B(D, C),
-  log_B(D, C),
   omega(D),
+  log_pi(D),
+  log_A(D),
+  log_B(D, C),
   log_omega(D),
   maxval(maxval),
   minval(
@@ -69,12 +69,14 @@ mnhmm::mnhmm(
 {
   for (arma::uword d = 0; d < D; ++d) {
     pi(d) = arma::vec(S);
-    log_pi(d) = arma::vec(S);
-    A(d) = arma::cube(S, S, Ti(0));
-    log_A(d) = arma::cube(S, S, Ti(0));
+    A(d) = arma::cube(S, S, Ti.max());
     for (arma::uword c = 0; c < C; ++c) {
-      B(d, c) = arma::cube(S, M(c) + 1, Ti(0));
-      log_B(d, c) = arma::cube(S, M(c) + 1, Ti(0));
+      B(d, c) = arma::cube(S, M(c) + 1, Ti.max());
+    }
+    log_pi(d) = arma::vec(S);
+    log_A(d) = arma::cube(S, S, Ti.max());
+    for (arma::uword c = 0; c < C; ++c) {
+      log_B(d, c) = arma::cube(S, M(c) + 1, Ti.max());
     }
   }
 }
@@ -83,83 +85,92 @@ void mnhmm::update_pi(const arma::uword i) {
   if (icpt_only_pi) {
     for (arma::uword d = 0; d < D; ++d) {
       pi(d) = softmax(gamma_pi(d).col(0));
-      log_pi(d) = arma::log(pi(d));
+      log_pi(d) = log_softmax(gamma_pi(d).col(0));
     }
   } else {
     for (arma::uword d = 0; d < D; ++d) {
-      pi(d) = softmax(gamma_pi(d) * X_pi.col(i));
-      log_pi(d) = arma::log(pi(d));
+      pi(d) = gamma_pi(d) * X_pi.col(i);
+      log_pi(d) = log_softmax(pi(d));
+      pi(d) = softmax(pi(d));
     }
   }
 }
 void mnhmm::update_A(const arma::uword i) {
-  for (arma::uword d = 0; d < D; ++d) {
-    A(d) = arma::cube(S, S, Ti(i));
-  }
   arma::mat Atmp(S, S);
+  arma::mat log_Atmp(S, S);
   if (icpt_only_A) {
     for (arma::uword d = 0; d < D; ++d) {
       for (arma::uword s = 0; s < S; ++s) { // from states
         Atmp.col(s) = softmax(gamma_A(d).slice(s).col(0));
+        log_Atmp.col(s) = log_softmax(gamma_A(d).slice(s).col(0));
       }
+      log_A(d).each_slice() = log_Atmp.t();
       A(d).each_slice() = Atmp.t();
-      log_A(d) = arma::log(A(d));
     }
   } else {
     for (arma::uword d = 0; d < D; ++d) {
       if (tv_A) {
         for (arma::uword t = 0; t < Ti(i); ++t) { // time
           for (arma::uword s = 0; s < S; ++s) { // from states
-            Atmp.col(s) = softmax(gamma_A(d).slice(s) * X_A(i).col(t));
+            Atmp.col(s) = gamma_A(d).slice(s) * X_A(i).col(t);
+            log_Atmp.col(s) = log_softmax(Atmp.col(s));
+            Atmp.col(s) = softmax(Atmp.col(s));
           }
+          log_A(d).slice(t) = log_Atmp.t();
           A(d).slice(t) = Atmp.t();
         }
       } else {
         for (arma::uword s = 0; s < S; ++s) { // from states
-          Atmp.col(s) = softmax(gamma_A(d).slice(s) * X_A(i).col(0));
+          Atmp.col(s) = gamma_A(d).slice(s) * X_A(i).col(0);
+          log_Atmp.col(s) = log_softmax(Atmp.col(s));
+          Atmp.col(s) = softmax(Atmp.col(s));
         }
+        log_A(d).each_slice() = log_Atmp.t();
         A(d).each_slice() = Atmp.t();
       }
-      log_A(d) = arma::log(A(d));
     }
   }
 }
 void mnhmm::update_B(const arma::uword i) {
   for (arma::uword c = 0; c < C; ++c) {
+    
+    arma::mat log_Btmp(M(c) + 1, S, arma::fill::zeros);
     arma::mat Btmp(M(c) + 1, S, arma::fill::ones);
-    for (arma::uword d = 0; d < D; ++d) {
-      B(d, c) = arma::cube(S, M(c) + 1, Ti(i));
-    }
     if (icpt_only_B(c)) {
       for (arma::uword d = 0; d < D; ++d) {
         for (arma::uword s = 0; s < S; ++s) { // from states
           Btmp.col(s).rows(0, M(c) - 1) = softmax(
             gamma_B(d, c).slice(s).col(0)
           );
+          log_Btmp.col(s).rows(0, M(c) - 1) = log_softmax(
+            gamma_B(d, c).slice(s).col(0)
+          );
         }
+        log_B(d, c).each_slice() = log_Btmp.t();
         B(d, c).each_slice() = Btmp.t();
-        log_B(d, c) = arma::log(B(d, c));
       }
     } else { 
       if (tv_B(c)) {
         for (arma::uword d = 0; d < D; ++d) {
           for (arma::uword t = 0; t < Ti(i); ++t) { // time
             for (arma::uword s = 0; s < S; ++s) { // from states
-              Btmp.col(s).rows(0, M(c) - 1) = softmax(gamma_B(d, c).slice(s) * X_B(c, i).col(t));
+              Btmp.col(s).rows(0, M(c) - 1) = gamma_B(d, c).slice(s) * X_B(c, i).col(t);
+              log_Btmp.col(s).rows(0, M(c) - 1) = log_softmax(Btmp.col(s).rows(0, M(c) - 1));
+              Btmp.col(s).rows(0, M(c) - 1) = softmax(Btmp.col(s).rows(0, M(c) - 1));
             }
+            log_B(d, c).slice(t) = log_Btmp.t();
             B(d, c).slice(t) = Btmp.t();
           }
-          log_B(d, c) = arma::log(B(d, c));
         }
       } else {
         for (arma::uword d = 0; d < D; ++d) {
           for (arma::uword s = 0; s < S; ++s) { // from states
-            Btmp.col(s).rows(0, M(c) - 1) = softmax(
-              gamma_B(d, c).slice(s) * X_B(c, i).col(0)
-            );
+            Btmp.col(s).rows(0, M(c) - 1) = gamma_B(d, c).slice(s) * X_B(c, i).col(0);
+            log_Btmp.col(s).rows(0, M(c) - 1) = log_softmax(Btmp.col(s).rows(0, M(c) - 1));
+            Btmp.col(s).rows(0, M(c) - 1) = softmax(Btmp.col(s).rows(0, M(c) - 1));
           }
+          log_B(d, c).each_slice() = log_Btmp.t();
           B(d, c).each_slice() = Btmp.t();
-          log_B(d, c) = arma::log(B(d, c));
         }
       }
     }
@@ -168,17 +179,19 @@ void mnhmm::update_B(const arma::uword i) {
 void mnhmm::update_omega(const arma::uword i) {
   if (icpt_only_omega) {
     omega = softmax(gamma_omega.col(0));
+    log_omega = log_softmax(gamma_omega.col(0));
   } else {
-    omega = softmax(gamma_omega * X_omega.col(i));  
+    omega = gamma_omega * X_omega.col(i);
+    log_omega = log_softmax(omega);
+    omega = softmax(omega);
   }
-  log_omega = arma::log(omega);
 }
-void mnhmm::update_log_py(const arma::uword i) {
-  log_py = arma::cube(S, Ti(i), D, arma::fill::zeros);
+void mnhmm::update_py(const arma::uword i) {
   for (arma::uword d = 0; d < D; ++d) {
+    py.slice(d).ones();
     for (arma::uword t = 0; t < Ti(i); ++t) {
       for (arma::uword c = 0; c < C; ++c) {
-        log_py.slice(d).col(t) += log_B(d, c).slice(t).col(obs(i)(c, t));
+        py.slice(d).col(t) %= B(d, c).slice(t).col(obs(i)(c, t));
       }
     }
   }
@@ -203,11 +216,12 @@ Rcpp::List mnhmm::viterbi() {
     if (!icpt_only_omega || i == 0) {
       update_omega(i);
     }
-    update_log_py(i);
+    update_py(i);
     arma::uvec q_d(Ti(i));
     for (arma::uword d = 0; d < D; ++d) {
       logp_d = univariate_viterbi(
-        q_d, log_omega(d) + log_pi(d), log_A(d), log_py.slice(d)
+        q_d, log_omega(d) + log_pi(d), log_A(d), 
+        arma::log(py.slice(d).cols(0, Ti(i) - 1))
       );
       if (logp_d > logp(i)) {
         logp(i) = logp_d;
@@ -223,8 +237,7 @@ Rcpp::List mnhmm::viterbi() {
 }
 
 arma::vec mnhmm::loglik() {
-  arma::vec log_alpha(S);
-  arma::vec log_alpha_new(S);
+  arma::vec alpha(S);
   arma::vec lli(D);
   arma::vec ll(N, arma::fill::zeros);
   for (arma::uword i = 0; i < N; ++i) {
@@ -240,18 +253,18 @@ arma::vec mnhmm::loglik() {
     if (!icpt_only_omega || i == 0) {
       update_omega(i);
     }
-    update_log_py(i);
+    update_py(i);
     for (arma::uword d = 0; d < D; ++d) {
-      log_alpha = log_pi(d) + log_py.slice(d).col(0);
+      alpha = pi(d) % py.slice(d).col(0);
+      double c = 1.0 / arma::accu(alpha);
+      alpha *= c;
+      lli(d) = -std::log(c);
       for (arma::uword t = 1; t < Ti(i); ++t) {
-        for (arma::uword s = 0; s < S; ++s) {
-          log_alpha_new(s) = logSumExp(
-            log_alpha + log_A(d).slice(t).col(s) + log_py(s, t, d)
-          );
-        }
-        log_alpha = log_alpha_new;
+        alpha = A(d).slice(t).t() * alpha % py.slice(d).col(t);
+        c = 1.0 / arma::accu(alpha);
+        alpha *= c;
+        lli(d) -= std::log(c);
       }
-      lli(d) = logSumExp(log_alpha);
     }
     ll(i) = logSumExp(log_omega + lli);
   }
@@ -274,12 +287,13 @@ arma::field<arma::mat> mnhmm::forward() {
     if (!icpt_only_omega || i == 0) {
       update_omega(i);
     }
-    update_log_py(i);
+    update_py(i);
     for (arma::uword d = 0; d < D; ++d) {
       arma::subview<double> submat = 
         log_alpha(i).rows(d * S, (d + 1) * S - 1);
-      univariate_forward(
-        submat, log_omega(d) + log_pi(d), log_A(d), log_py.slice(d)
+      univariate_forward_logspace(
+        submat, log_omega(d) + log_pi(d), log_A(d), 
+        arma::log(py.slice(d).cols(0, Ti(i) - 1))
       );
     }
   }
@@ -296,11 +310,14 @@ arma::field<arma::mat> mnhmm::backward() {
     if (arma::any(iv_B) || i == 0) {
       update_B(i);
     }
-    update_log_py(i);
+    update_py(i);
     for (arma::uword d = 0; d < D; ++d) {
       arma::subview<double> submat = 
         log_beta(i).rows(d * S, (d + 1) * S - 1);
-      univariate_backward(submat, log_A(d), log_py.slice(d));
+      univariate_backward_logspace(
+        submat, log_A(d),
+        arma::log(py.slice(d).cols(0, Ti(i) - 1))
+      );
     }
   }
   return log_beta;
@@ -416,96 +433,89 @@ Rcpp::List mnhmm::simulate() {
   );
 }
 void mnhmm::gradient_omega(
-    arma::mat& grad, arma::mat& tmpmat,
-    const arma::vec& loglik_i, const arma::vec& loglik,
-    const arma::uword i) {
-  
-  tmpmat = -omega * omega.t();
-  tmpmat.diag() += omega;
-  grad += tmpmat * exp(loglik_i - loglik(i)) * X_omega.col(i).t();
+    arma::mat& grad, const arma::vec& pcp, const arma::uword i) {
+  grad += (omega % pcp - omega * arma::dot(omega, pcp)) * X_omega.col(i).t();
 }
 void mnhmm::gradient_pi(
     arma::mat& grad, 
-    arma::mat& tmpmat, 
-    const arma::cube& log_beta, 
-    const arma::vec& loglik, 
+    arma::vec& tmpvec, 
+    const arma::vec& pcp,
+    const arma::mat& beta, 
     const arma::uword i,
     const arma::uword d) {
   
-  tmpmat = -pi(d) * pi(d).t();
-  tmpmat.diag() += pi(d);
-  grad += tmpmat * exp(log_omega(d) + log_py.slice(d).col(0) + 
-    log_beta.slice(d).col(0) - loglik(i)) * X_pi.col(i).t();
+  // tmpvec = omega(d) * py.slice(d).col(0) % beta.col(0);
+  tmpvec = py.slice(d).col(0) % beta.col(0);
+  grad += pcp(d) * (pi(d) % tmpvec - pi(d) * arma::dot(pi(d), tmpvec)) * X_pi.col(i).t();
 }
+
 void mnhmm::gradient_A(
     arma::mat& grad, 
-    arma::mat& tmpmat,
-    const arma::cube& log_alpha, 
-    const arma::cube& log_beta, 
-    const arma::vec& loglik, 
+    arma::vec& tmpvec1,
+    arma::vec& tmpvec2,
+    const arma::vec& pcp,
+    const arma::mat& alpha, 
+    const arma::mat& beta, 
     const arma::uword i, 
     const arma::uword t, 
     const arma::uword s, 
     const arma::uword d) {
   
-  tmpmat = -A(d).slice(t).row(s).t() * A(d).slice(t).row(s);
-  tmpmat.diag() += A(d).slice(t).row(s);
-  grad += tmpmat * exp(log_omega(d) + log_alpha(s, t - 1, d) + 
-    log_py.slice(d).col(t) + log_beta.slice(d).col(t) - 
-    loglik(i)) * X_A(i).col(t).t();
+  tmpvec1 = A(d).slice(t).row(s).t();
+  tmpvec2 = py.slice(d).col(t) % beta.col(t) * alpha(s, t - 1);
+  // grad += omega(d) * (tmpvec1 % tmpvec2 - tmpvec1 * 
+  //   arma::dot(tmpvec1, tmpvec2)) * X_A(i).col(t).t();
+  grad += pcp(d) * (tmpvec1 % tmpvec2 - tmpvec1 * 
+    arma::dot(tmpvec1, tmpvec2)) * X_A(i).col(t).t();
 }
 void mnhmm::gradient_B_t1(
     arma::mat& grad, 
     arma::vec& tmpvec, 
-    const arma::cube& log_beta, 
-    const arma::vec& loglik, 
+    const arma::vec& pcp,
+    const arma::mat& beta, 
     const arma::uword i, 
     const arma::uword s, 
     const arma::uword c, 
     const arma::uword d) {
   
-  arma::uword C = M.n_elem;
-  arma::rowvec Brow = B(d, c).slice(0).row(s).cols(0, M(c) - 1);
-  arma::uword idx = obs(i)(c, 0);
-  double brow = Brow(idx);
+  arma::subview_row<double> Brow = B(d, c).slice(0).row(s).cols(0, M(c) - 1);
+  const arma::uword idx = obs(i)(c, 0);
+  const double brow = Brow(idx);
   tmpvec = -Brow.t() * brow;
   tmpvec(idx) += brow;
-  double logpy = 0;
+  double pyx = 1;
   for (arma::uword cc = 0; cc < C; ++cc) {
     if (cc != c) {
-      logpy += log_B(d, cc)(s, obs(i)(cc, 0), 0);
+      pyx *= B(d, cc)(s, obs(i)(cc, 0), 0);
     }
   }
-  grad += exp(log_omega(d) + log_pi(d)(s) + logpy + 
-    log_beta(s, 0, d) - loglik(i)) * tmpvec * X_B(c, i).col(0).t();
+  grad += pcp(d) * pi(d)(s) * pyx * beta(s, 0) * tmpvec * X_B(c, i).col(0).t();
 }
 void mnhmm::gradient_B(
     arma::mat& grad, 
     arma::vec& tmpvec, 
-    const arma::cube& log_alpha, 
-    const arma::cube& log_beta, 
-    const arma::vec& loglik, 
+    const arma::vec& pcp,
+    const arma::mat& alpha, 
+    const arma::mat& beta, 
     const arma::uword i, 
     const arma::uword s, 
     const arma::uword t, 
     const arma::uword c,
     const arma::uword d) {
   
-  arma::uword C = M.n_elem;
-  arma::rowvec Brow = B(d, c).slice(t).row(s).cols(0, M(c) - 1);
-  arma::uword idx = obs(i)(c, t);
-  double brow = Brow(idx);
+  arma::subview_row<double> Brow = B(d, c).slice(t).row(s).cols(0, M(c) - 1);
+  const arma::uword idx = obs(i)(c, t);
+  const double brow = Brow(idx);
   tmpvec = -Brow.t() * brow;
   tmpvec(idx) += brow;
-  double logpy = 0;
+  double pyx = 1;
   for (arma::uword cc = 0; cc < C; ++cc) {
     if (cc != c) {
-      logpy += log_B(d, cc)(s, obs(i)(cc, t), t);
+      pyx *= B(d, cc)(s, obs(i)(cc, t), t);
     }
   }
-  grad += arma::accu(exp(log_omega(d) + log_alpha.slice(d).col(t - 1) + 
-    log_A(d).slice(t).col(s) + logpy + log_beta(s, t, d) - loglik(i))) * 
-    tmpvec * X_B(c, i).col(t).t();
+  grad += pcp(d) * arma::dot(alpha.col(t - 1), A(d).slice(t).col(s)) * 
+    beta(s, t) * pyx * tmpvec * X_B(c, i).col(t).t();
 }
 
 Rcpp::List mnhmm::log_objective(const arma::mat& Qs, 
@@ -529,8 +539,9 @@ Rcpp::List mnhmm::log_objective(const arma::mat& Qs,
   arma::field<arma::cube> grad_B2(C, D);
   arma::vec loglik(N);
   arma::vec loglik_i(D);
-  arma::cube log_alpha(S, Ti(0), D);
-  arma::cube log_beta(S, Ti(0), D);
+  arma::cube alpha(S, Ti.max(), D);
+  arma::cube beta(S, Ti.max(), D);
+  arma::vec scales(Ti.max());
   for (arma::uword d = 0; d < D; ++d) {
     grad_pi2(d) = arma::mat(S - 1, X_pi.n_rows, arma::fill::zeros);
     grad_A2(d) = arma::cube(S - 1, X_A(0).n_rows, S, arma::fill::zeros);
@@ -538,11 +549,11 @@ Rcpp::List mnhmm::log_objective(const arma::mat& Qs,
       grad_B2(c, d) = arma::cube(M(c) - 1, X_B(c, 0).n_rows, S, arma::fill::zeros);
     }
   }
-  arma::mat tmpmat(S, S);
-  arma::mat tmpmatD(D, D);
-  arma::field<arma::vec> tmpvec(C);
+  arma::vec tmpvecS1(S);
+  arma::vec tmpvecS2(S);
+  arma::field<arma::vec> tmpvecM(C);
   for (arma::uword c = 0; c < C; ++c) {
-    tmpvec(c) = arma::vec(M(c));
+    tmpvecM(c) = arma::vec(M(c));
   }
   for (arma::uword i = 0; i < N; ++i) {
     if (!icpt_only_pi || i == 0) {
@@ -557,15 +568,10 @@ Rcpp::List mnhmm::log_objective(const arma::mat& Qs,
     if (!icpt_only_omega || i == 0) {
       update_omega(i);
     }
-    update_log_py(i);
-    log_alpha = arma::cube(S, Ti(i), D);
-    log_beta = arma::cube(S, Ti(i), D);
+    update_py(i);
     for (arma::uword d = 0; d < D; ++d) {
-      univariate_forward(
-        log_alpha.slice(d), log_pi(d), log_A(d), log_py.slice(d)
-      );
-      univariate_backward(log_beta.slice(d), log_A(d), log_py.slice(d));
-      loglik_i(d) = logSumExp(log_alpha.slice(d).col(Ti(i) - 1));
+      loglik_i(d) = univariate_forward(alpha.slice(d), scales, pi(d), A(d), py.slice(d), Ti(i));
+      univariate_backward(beta.slice(d), scales, A(d), py.slice(d), Ti(i));
     }
     loglik(i) = logSumExp(log_omega + loglik_i);
     if (!std::isfinite(loglik(i))) {
@@ -579,14 +585,16 @@ Rcpp::List mnhmm::log_objective(const arma::mat& Qs,
     }
     // gradient wrt gamma_pi
     // d loglik / d pi
+    arma::vec pcp = omega % arma::exp(loglik_i - loglik(i));
     for (arma::uword d = 0; d < D; ++d) {
       // gradient wrt gamma_pi
-      gradient_pi(grad_pi(d), tmpmat, log_beta, loglik, i, d );
+      gradient_pi(grad_pi(d), tmpvecS1, pcp, beta.slice(d), i, d);
       // gradient wrt gamma_A
       for (arma::uword t = 1; t < Ti(i); ++t) {
         for (arma::uword s = 0; s < S; ++s) {
           gradient_A(
-            grad_A(d).slice(s), tmpmat, log_alpha, log_beta, loglik, i, t, s, d
+            grad_A(d).slice(s), tmpvecS1, tmpvecS2, pcp, alpha.slice(d), 
+            beta.slice(d), i, t, s, d
           );
         }
       }
@@ -595,23 +603,21 @@ Rcpp::List mnhmm::log_objective(const arma::mat& Qs,
         for (arma::uword s = 0; s < S; ++s) {
           if (obs(i)(c, 0) < M(c)) {
             gradient_B_t1(
-              grad_B(c, d).slice(s), tmpvec(c), log_beta, loglik, i, s, c, d
+              grad_B(c, d).slice(s), tmpvecM(c), pcp, beta.slice(d), i, s, c, d
             );
           }
           for (arma::uword t = 1; t < Ti(i); ++t) {
             if (obs(i)(c, t) < M(c)) {
               gradient_B(
-                grad_B(c, d).slice(s), tmpvec(c), log_alpha, log_beta, loglik, 
-                i, s, t, c, d
+                grad_B(c, d).slice(s), tmpvecM(c), pcp, alpha.slice(d), 
+                beta.slice(d), i, s, t, c, d
               );
             }
           }
         }
       }
     }
-    gradient_omega(
-      grad_omega, tmpmatD, loglik_i, loglik, i
-    );
+    gradient_omega(grad_omega, arma::exp(loglik_i - loglik(i)), i);
   }
   grad_omega2 = Qd.t() * grad_omega;
   for (arma::uword d = 0; d < D; ++d) {
