@@ -1,8 +1,8 @@
 #ifndef REPARMA_H
 #define REPARMA_H
 
+#include "config.h"
 #include <algorithm>
-#include <RcppArmadillo.h>
 arma::vec reparma(const arma::vec& x, const arma::uvec& y);
 
 #endif
